local deformat = AceLibrary("Deformat-2.0");
local YataTT = AceLibrary("Gratuity-2.0");
local Yata_TotemTiming = {Yata.L["min"], Yata.L["sec"]};
-- potential Blacklist to get around spells with the Totemstring and which aren't Totems at all
local Yata_SpellBlacklist = {Yata.L["Totemic Call"]};



----------
-- Init --
----------

function Yata:TotemsInit()
	self.ElementsCount = {["Earth"] = 0, ["Air"] = 0, ["Water"] = 0, ["Fire"] = 0};
	if(not self.db.profile.TotemsOrder) then
		self.db.profile.TotemsOrder = {"Earth","Air","Water","Fire"};
		for i=1,4 do
			self.db.profile.TotemsOrder[self.db.profile.TotemsOrder[i]] = {};
		end
	end
	self:GatherTotemData();
	self:CleanTotemProfile();
end
---------------------------
-- Get Position by totem --
---------------------------

function Yata:GetPosByTotem(totem)
	local retval = 0;
	if(self.db.profile.TotemsOrder[totem.Element]) then
		for k,v in ipairs(self.db.profile.TotemsOrder[totem.Element]) do
			if(v == totem.Name) then retval = k;
				break;
			end
		end
	end
	return retval;
end

-----------------------------
-- Return order by element --
-----------------------------

function Yata:ReturnOrderByElement(ele)
	local retval = 0;
	for i=1,4 do
		if(self.db.profile.TotemsOrder[i] == ele) then
			retval = i;
			break;
		end
	end
	return retval;
end

-----------------------
-- CleanTotemProfile --
-----------------------

function Yata:CleanTotemProfile()
	if(self.TotemData.Count ~= self.db.profile.TotemsCount) then
		for i=1,4 do
			for k,v in ipairs(self.db.profile.TotemsOrder[self.db.profile.TotemsOrder[i]]) do
				if(not self.TotemData[v]) then 
					self:Print("Couldn't find: "..v..", removing it");
					table.remove(self.db.profile.TotemsOrder[self.db.profile.TotemsOrder[i]],k);
				end
			end
		end
	end
end


-----------------------------
-- Utility functions below --
-----------------------------
function Yata:SpellBlacklist(spell)
	local blacklist = nil;
	for k,v in ipairs(Yata_SpellBlacklist) do
		if(spell == v) then
			blacklist = true;
			break;
		end
	end
	return blacklist
end

function Yata:GatherTotemData()
    self.TotemData = {};
    local tabs = GetNumSpellTabs();
    for tab = 2, tabs do
	local _, _, offset, numSpells = GetSpellTabInfo(tab);
	for i = 1, numSpells do
	    local spell = offset + i;
	    local SpellName, subspell = GetSpellName(spell, "spell");
	    if (string.find(string.lower(SpellName), string.lower(Yata.L["Totem"])) and not self:SpellBlacklist(SpellName)) then
		if (not self.TotemData[SpellName]) then
		    self.TotemData[SpellName] = {};
		    table.insert(self.TotemData, SpellName);
		end
		local data = {};
		data.SpellID = spell;
		table.insert(self.TotemData[SpellName],data);
	    end
	end
    end
    local count = 0;
    -- Add Spell Information (Cooldown,Duration, HP, Element)
    	for k,v in ipairs(self.TotemData) do
    		for l,w in ipairs(self.TotemData[v]) do
			local data = self:ExtractToolTip(w.SpellID);
			w.Element = data.element;
			w.Cooldown = data.Cooldown;
			w.Health = data.health;
			w.Duration = data.duration;
			w.Mana = data.mana;
			w.Name = v;
			w.Texture = GetSpellTexture(w.SpellID,"spell");
		end
		count = count + 1;
	end
    self.TotemData.Count = count;
end
--- Return Cooldown, Duration, HP, Element
--- Not yet fully converted to Gratuity/Deformat
function Yata:ExtractToolTip(spellid)
	local element = "";
	local data = {};
	data.element = "";
	data.Cooldown = 0;
	data.health = 0;
	data.duration = 0;
	data.mana = 0;
	YataTT:SetSpell( spellid, "spell" );
	-- Mana cost --
	data.mana = YataTT:FindDeformat(MANA_COST,2,2,false,true);
	-- Element extraction --
	local elesearch = Yata.L["Tools"]..": (.*)"..Yata.L["totem"];
	local _,_, element = YataTT:Find(elesearch,4,4,false,true);
	if( element ) then	
		if( string.sub( element, 1, 1 ) == "|" ) then
			element = string.sub( element, 11 );
		end
		_, _, element = string.find(element, "(%a*)");
		-- localization hacks incoming --
		if( GetLocale() == "deDE" ) then
			if(element == "Erd") then
				element = "Erde"; -- "Werkzeuge: Erdtotem", rename Erd to Erde
			end
		end
		data.element = Yata.L:GetReverseTranslation(element);
		if(not data.element) then
			self:Debug("Localization bug incoming");
		end
	end
	-- End Element --
	text = YataTT:GetLine(5,nil);
	-- Start HP --
	local _, _, health = YataTT:Find("(%d+) "..Yata.L["health"],5,5,false,true);
	if(not health) then
		_, _, health = YataTT:Find("(%d+) "..Yata.L["points"].." "..Yata.L["health"],5,5,false,true);
	end
	data.health = health;
	-- End HP --
	-- Start Duration --     
	     local matches = {};
	     for num, regex in ipairs(Yata_TotemTiming) do
		     for k in string.gmatch(text,"(%d+) "..regex) do
	       		if(num == 1) then
	       			table.insert(matches,tonumber(k*60));
	       		else 	table.insert(matches,tonumber(k));
	       		end
		     end
	     end
	     table.sort(matches);
	     data.duration = matches[table.getn(matches)];
	-- End Duration --
	-- Start Cooldown -- 
	local matches = {};
	text = YataTT:GetLine(3,true);
	if(text) then
	     for num, regex in ipairs(Yata_TotemTiming) do
		     for k in string.gmatch(text,"(%d+) "..regex) do
	       		if(num == 1) then
	       			table.insert(matches,tonumber(k*60));
	       		else 	table.insert(matches,tonumber(k));
	       		end
		     end
	     end
	     table.sort(matches);
	     data.Cooldown = matches[table.getn(matches)];
	end
	-- End Cooldown --
	return data;
end
function Yata:OutputTotemData()
	for k,v in ipairs(self.TotemData) do
		for l,w in ipairs(self.TotemData[v]) do
			self:Print(v.." ".."(Rank "..l.."), HP: "..w.Health..", Duration: "..w.Duration..", Element: "..w.Element..", Cooldown: "..w.Cooldown..", Mana: "..w.Mana);
		end
	end
end